import torch
from torch import nn
from torch.utils.data import DataLoader
from torchvision import datasets
from torchvision.transforms import ToTensor
import torch.nn.functional as F


# Download training data from open datasets.
training_data = datasets.MNIST(
	root="data",
	train=True,
	download=False,
	transform=ToTensor(),
)

# Download test data from open datasets.
test_data = datasets.MNIST(
	root="data",
	train=False,
	download=False,
	transform=ToTensor(),
)


batch_size = 64

# Create data loaders.
train_dataloader = DataLoader(training_data, batch_size=batch_size)
test_dataloader = DataLoader(test_data, batch_size=batch_size)

for X, y in test_dataloader:
	print(f"Shape of X [N, C, H, W]: {X.shape}")
	print(f"Shape of y: {y.shape} {y.dtype}")
	break

# Define model
class DeepNet(nn.Module):
	def __init__(self):
		super(DeepNet, self).__init__()
		# Capa 1: 28*28
		self.conv1 = nn.Conv2d( 1, 32, 3, 1 )
		self.act1  = nn.ReLU()

		# Capa 2: 26*26*32
		self.conv2 = nn.Conv2d( 32, 32, 3, 1 )
		self.act2  = nn.ReLU()

		# Capa 3: 24*24*32 = 18432
		self.pooling = nn.MaxPool2d( 2, stride=2 ) 
		self.drop1  = nn.Dropout(0.25)
		# Capa 3: Salida: 12*12*32 = 4608

		# Capa 4. Totalmente conectada
		self.flatten = nn.Flatten()
		self.linear_relu_stack = nn.Sequential(
			nn.Linear( 4608, 128 ),
			nn.ReLU(),
			nn.Dropout(0.5),

			# Capa 5. Totalmente conectada
			nn.Linear( 128, 10 ),
		)


	def forward(self, x):
		x = self.conv1(x)
		x = self.act1(x)
		x = self.conv2(x)
		x = self.act2(x)
		x = self.pooling(x)
		x = self.drop1(x)
		x = self.flatten(x )
		x = self.linear_relu_stack(x)
		salida = F.log_softmax(x, dim=1)
		return  salida


# modelo = DeepNet().to(device)
modelo = DeepNet()
print(modelo)

# Equates to one random 28x28 image
imagen_aleatoria = torch.rand((1, 1, 28, 28))

resultado = modelo( imagen_aleatoria )
print (resultado)
